unit comlcd;
{
  Serial LCD unit for MPXF Software
  Written on 10/16/99

}
interface
procedure SInitLCD;
procedure SClear;
procedure SClose;
procedure SCursorOn;
procedure SCursorOff;
procedure SHome;
procedure SPushChar(ch:char);
procedure SCursor(l,x,lcdlen:byte);
procedure SerialReadSettings(section:string);
procedure SDefineChar(cn,d1,d2,d3,d4,d5,d6,d7,d8:byte);

implementation
uses crt,config,dos;
const  coms       = 4;     { Anzahl der untersttzten Schnittstellen }

       ua         : array[1..coms] of word = ($3f8,$2f8,$3e8,$2e8);
       datainout  = 0;     { UART-Register-Offsets }
       intenable  = 1;
       intids     = 2;     { Read  }
       fifoctrl   = 2;     { Write }
       linectrl   = 3;
       modemctrl  = 4;
       linestat   = 5;
       modemstat  = 6;
       scratch    = 7;

       UartNone   = 0;     { Ergebnisse von ComType }
       Uart8250   = 1;
       Uart16450  = 2;
       Uart16550  = 3;
       Uart16550A = 4;

       NoFifo     = $00;   { Triggerlevel bei 16550-Chips }
       FifoTL1    = $07;
       FifoTL4    = $47;
       FifoTL8    = $87;
       FifoTL14   = $C7;

type   paritype   = (Pnone,Podd,Pxxxx,Peven);   { mgliche Paritts-Typen }

type
 PinStatus = byte;
 PinData = array[1..17] of PinStatus;

 const
   PinDataOn:Pindata = (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0);
   PinDataOff:pindata = (1,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,8);

var
  CommandChar,
  CursorHome,
HideCursor,
ShowCursor,
startrow,startcolumn,
Formfeed,
gotoxyoffset,
Backlighton,
Backlightoff,
blinkon,blinkoff,
stopbits,
cmdgotoxy,
CCOffset1,
CCOffset2,
CustomCharStart: byte;
customchar:string[12];
UseCustom:boolean;
InitString:string[40];
usenormalcustom:boolean;


const  active     : array[1..coms] of boolean = (false,false,false,false);
       irq        : array[1..coms] of byte = ($04,$03,0,0);
       intmask    : array[1..coms] of byte = ($10,$08,0,0);
       intcom2    : array[1..coms] of boolean = (false,false,false,false);

       MS_CTS     = $10;       { Modem-Status-Register }
       MS_DSR     = $20;
       MS_RI      = $40;       { Ring Indicator: Klingelsignal }
       MS_DCD     = $80;       { Data Carrier Detect           }
       MC_DTR     = $01;       { Modem Control Register }
       MC_RTS     = $02;

type   bufft      = array[0..65534] of byte;

var    savecom    : array[1..coms] of pointer;
{       exitsave   : pointer;}
       bufsize    : array[1..coms] of word;
       buffer     : array[1..coms] of ^bufft;
       bufi,bufo  : array[1..coms] of word;
       setup:boolean;
       i:integer;
       comport:word;
       baudrate:word;



procedure error(text:string);
begin
  writeln('UART Error: ',text);
end;

procedure sendcommand(no:byte);
begin
  if commandchar <> 0 then begin
    while (port[ua[no]+linestat] and $20) = 0 do;
    port[ua[no]]:=CommandChar;
  end;
end;


procedure sendbyte(no,b:byte);              { Byte senden }
begin
  sendcommand(no);
  while (port[ua[no]+linestat] and $20) = 0 do;
  port[ua[no]]:=b;
end;

procedure sendbyte2(no,b:byte);              { Byte senden }
begin
  while (port[ua[no]+linestat] and $20) = 0 do;
  port[ua[no]]:=b;
end;

procedure sendchar(no,b:byte);              { Byte senden }
begin
  if ((b >= 0) and (b <=7)) and (not UseCustom) then exit;
    if ((b >= 0) and (b <= 7)) then
      b:=b+customcharstart;
  while (port[ua[no]+linestat] and $20) = 0 do;
  port[ua[no]]:=b;
end;


procedure flushinput(no:byte);            { Receive-Puffer lschen }
begin
  bufo[no]:=bufi[no];
end;

procedure com1server; interrupt;
begin
  if intcom2[1] then port[$a0]:=$20;
  port[$20]:=$20;                      { Interrupt-Controller resetten }
  buffer[1]^[bufi[1]]:=port[ua[1]];
  inc(bufi[1]); if bufi[1]=bufsize[1] then bufi[1]:=0;
end;

procedure com2server; interrupt;
begin
  if intcom2[2] then port[$a0]:=$20;
  port[$20]:=$20;
  buffer[2]^[bufi[2]]:=port[ua[2]];
  inc(bufi[2]); if bufi[2]=bufsize[2] then bufi[2]:=0;
end;

procedure com3server; interrupt;
begin
  if intcom2[3] then port[$a0]:=$20;
  port[$20]:=$20;
  buffer[3]^[bufi[3]]:=port[ua[3]];
  inc(bufi[3]); if bufi[3]=bufsize[3] then bufi[3]:=0;
end;

procedure com4server; interrupt;
begin
  if intcom2[4] then port[$a0]:=$20;
  port[$20]:=$20;
  buffer[4]^[bufi[4]]:=port[ua[4]];
  inc(bufi[4]); if bufi[4]=bufsize[4] then bufi[4]:=0;
end;

procedure com1FIFOserver; interrupt;
begin
  if port[ua[1]+intids] and 4<>0 then
    repeat
      buffer[1]^[bufi[1]]:=port[ua[1]];
      inc(bufi[1]); if bufi[1]=bufsize[1] then bufi[1]:=0;
    until not odd(port[ua[1]+linestat]);
  if intcom2[1] then port[$a0]:=$20;
  port[$20]:=$20;                      { Interrupt-Controller resetten }
end;

procedure com2FIFOserver; interrupt;
begin
  if port[ua[2]+intids] and 4<>0 then
    repeat
      buffer[2]^[bufi[2]]:=port[ua[2]];
      inc(bufi[2]); if bufi[2]=bufsize[2] then bufi[2]:=0;
    until not odd(port[ua[2]+linestat]);
  if intcom2[2] then port[$a0]:=$20;
  port[$20]:=$20;
end;

procedure com3FIFOserver; interrupt;
begin
  if port[ua[3]+intids] and 4<>0 then
    repeat
      buffer[3]^[bufi[3]]:=port[ua[3]];
      inc(bufi[3]); if bufi[3]=bufsize[3] then bufi[3]:=0;
    until not odd(port[ua[3]+linestat]);
  if intcom2[3] then port[$a0]:=$20;
  port[$20]:=$20;
end;

procedure com4FIFOserver; interrupt;
begin
  if port[ua[4]+intids] and 4<>0 then
    repeat
      buffer[4]^[bufi[4]]:=port[ua[4]];
      inc(bufi[4]); if bufi[4]=bufsize[4] then bufi[4]:=0;
    until not odd(port[ua[4]+linestat]);
  if intcom2[4] then port[$a0]:=$20;
  port[$20]:=$20;
end;

procedure DropDtr;                 { RTS=0 setzen      }
begin
  port[ua[comport]+modemctrl]:=port[ua[comport]+modemctrl] and (not MC_DTR);
end;

procedure SetDtr;                  { RTS=1 setzen      }
begin
  port[ua[comport]+modemctrl]:=port[ua[comport]+modemctrl] or MC_DTR;
end;

procedure DropCts;                 { RTS=0 setzen      }
begin
  port[ua[comport]+modemctrl]:=port[ua[comport]+modemctrl] and (not MS_CTS);
end;

procedure SetCts;                  { RTS=1 setzen      }
begin
  port[ua[comport]+modemctrl]:=port[ua[comport]+modemctrl] or MS_CTS;
end;

procedure SPushChar(ch:char);
  begin
    sendchar(comport,ord(ch));
  end;

procedure SDefineChar(cn,d1,d2,d3,d4,d5,d6,d7,d8:byte);
var
 a:string;
       b:string;
      cd:byte;
      c:integer;
      pp:byte;

  begin
    if UseCustom then
    begin
      sendcommand(comport);
      if usenormalcustom then sendbyte2(comport,$40+(cn shl 3))
      else
      begin
        a:='#'+customchar;

        while (a[1] = '#') do
        begin
          a:=copy(a,2,length(a));
          pp := pos('#',a);
          if pp = 0 then pp := length(a)+1;
          b:= copy(a,1,pp-1);
          val(b,cd,c);
  {        writeln('Send ',cd);}
          sendbyte2(comport,cd);
          delay(10);
          a:=copy(a,pp,length(a));
        end;
{      readln;}
      end;

{      sendbyte2(comport,customchar);  { Define Char }

      sendbyte2(comport,CCOffset1+cn);
      sendbyte2(comport,CCOffset2+d1);
      sendbyte2(comport,CCOffset2+d2);
      sendbyte2(comport,CCOffset2+d3);
      sendbyte2(comport,CCOffset2+d4);
      sendbyte2(comport,CCOffset2+d5);
      sendbyte2(comport,CCOffset2+d6);
      sendbyte2(comport,CCOffset2+d7);
      sendbyte2(comport,CCOffset2+d8);
    end;
  end;


procedure SCursor(l,x,lcdlen:byte);
  begin
    if gotoxyoffset <> 0 then
    begin
      sendcommand(comport);
      if cmdgotoxy <> 0 then sendbyte2(comport,cmdgotoxy);
      sendbyte2(comport, gotoxyoffset +((lcdlen)*(l-startrow))+(x-startcolumn));
    end else
    begin
      sendcommand(comport);
      sendbyte2(comport,cmdgotoxy);
      sendbyte2(comport,x-1+startcolumn);
      sendbyte2(comport,l-1+startrow);
    end;
  end;

procedure SerialReadSettings(section:string);
begin
  usenormalcustom := ReadBoolean(section,'UseNormalCustom');
  Comport := ReadWord(section,'Comport');
  BaudRate := ReadWord(section,'BaudRate');
  StopBits := ReadWord(section,'Stopbits');
  UseCustom := ReadBoolean(section,'DisableCustomChar');
  UseCustom := not UseCustom;
  if stopbits = 0 then stopbits := 1;
  InitString := ReadString(section,'InitString');

  CommandChar:=ReadWord(section,'CommandChar');
  gotoxyoffset:=readword(section,'GotoXYOffset');
  cmdgotoxy:=readword(section,'GotoXY');
  CursorHome:=ReadWord(section,'CursorHome');
  StartRow:=ReadWord(section,'StartRow');
  StartColumn:=ReadWord(section,'StartColumn');
  if gotoxyoffset <> 0 then
  begin
    if StartRow = 0 then StartRow := 1 else Startrow := 0;
    if StartColumn = 0 then StartColumn := 1 else StartColumn := 0;
  end;
  blinkon:=readword(section,'blinkOn');
  blinkoff:=readword(section,'blinkoff');
  HideCursor:=ReadWord(section,'HideCursor');
  ShowCursor:=ReadWord(section,'ShowCursor');
  Formfeed:=ReadWord(section,'ClearScreen');
  Backlighton:=Readword(section,'Backlighton');
  Backlightoff:=ReadWord(section,'Backlightoff');
  CustomCharStart:=ReadWord(section,'CustomCharStart');
  CCOffset1 := ReadWord(section,'CustomCharOffset1');
  CCOffset2 := ReadWord(section,'CustomCharOffset2');
  CustomChar:=ReadString(section,'CustomChar');
end;


function strs(l:longint):string;
var s : string;
begin
  str(l,s);
  strs:=s;
end;

procedure clearstatus(no:byte);
begin
  if port[ua[no]+datainout]<>0 then;               { dummy-Read }
  if port[ua[no]+linestat]<>0 then;
  if port[ua[no]+modemstat]<>0 then;
  if intcom2[no] then port[$a0]:=$20;
  port[$20]:=$20;
end;

function IntNr(no:byte):byte;
begin
  if irq[no]<8 then IntNr:=irq[no]+8
  else IntNr:=irq[no]+$68;
end;

procedure setuart(comno:byte; baudrate:longint; parity:paritype;
                  wlength,stops:byte);
var uart : word;
begin
  uart:=ua[comno];
  port[uart+linectrl]:=$80;
  port[uart+datainout]:=lo(word(115200 div baudrate));
  port[uart+datainout+1]:=hi(word(115200 div baudrate));
  port[uart+linectrl]:= (wlength-5) or (stops-1)*4 or ord(parity)*8;
  port[uart+modemctrl]:=$0b;
  if port[uart+datainout]<>0 then;      { dummy }
end;

procedure ActivateCom(no:byte; buffersize:word; FifoTL:Byte);
var p : pointer;
    i : byte;
begin
  if active[no] then begin
    error('Error activating comport '+strs(no)+'!');
    exit;
    end
  else if (no<1) or (no>coms) or (irq[no]=0) then
    error('Comport '+strs(no)+' is not a valid comport!')
  else
    active[no]:=true;

  bufsize[no]:=buffersize;                 { Puffer anlegen }
  getmem(buffer[no],buffersize);
  bufi[no]:=0; bufo[no]:=0;
  fillchar(buffer[no]^,bufsize[no],0);

  IF (fifotl > 0)
    THEN BEGIN
           Port[(ua[no] + fifoctrl)] := fifotl;
           IF ((Port[(ua[no] + intids)] AND $40) = 0)
             THEN BEGIN
                    Port[(ua[no] + fifoctrl)] := 0;
                    fifotl := NoFifo;
                  END;
         END;

  IF (fifotl > 0)
    THEN CASE no OF
           1 : p:=@com1FIFOserver;
           2 : p:=@com2FIFOserver;
           3 : p:=@com3FIFOserver;
           4 : p:=@com4FIFOserver;
         END {CASE}
    ELSE CASE no OF
           1 : p:=@com1server;
           2 : p:=@com2server;
           3 : p:=@com3server;
           4 : p:=@com4server;
         END; {CASE}

  getintvec(IntNr(no),savecom[no]);           { IRQ setzen }
  setintvec(IntNr(no),p);
  port[ua[no]+intenable]:=$01;                     { Int. bei Empfang }
  if intcom2[no] then
    port[$a1]:=port[$a1] and (not intmask[no])     { Ints freigeben }
  else
    port[$21]:=port[$21] and (not intmask[no]);
  clearstatus(no);
end;

procedure releasecom(no:byte);
begin
  if not active[no] then
    error('Error releaseing comport '+strs(no)+'!')
  else begin
    active[no]:=false;
    port[ua[no]+intenable]:=0;
    if intcom2[no] then
      port[$a1]:=port[$a1] or intmask[no]    { Controller: COMn-Ints sperren }
    else
      port[$21]:=port[$21] or intmask[no];
    port[ua[no]+fifoctrl]:=0;
    setintvec(IntNr(no),savecom[no]);
    clearstatus(no);
    freemem(buffer[no],bufsize[no]);
    end;
end;

procedure SClear;
  begin
    sendbyte(comport, formfeed);
  end;

procedure SCursorOn;
  begin
    sendbyte(comport, ShowCursor);
    sendbyte(comport, BlinkOn);
  end;

procedure SCursorOff;
  begin
    sendbyte(comport, HideCursor);
    sendbyte(comport, BlinkOff);
  end;

procedure SHome;
  begin
    sendbyte(comport, CursorHome);
  end;

procedure SInitLCD;
  var p:paritype;
      a:string;
      b:string;
      cd:byte;
      c:integer;
      pp:byte;
begin

  if not active[comport] then
  begin
    p := Pnone;
    setuart(comport,baudrate,p,8,stopbits);
    activatecom(comport,100,0);
{    writeln('Serial LCD open for com',comport,' baudrate',baudrate);}
  end;

  dropdtr;
  delay(200);
  setdtr;
  a := InitString;
  while (a[1] = '#') do
  begin
    a:=copy(a,2,length(a));
    pp := pos('#',a);
    if pp = 0 then pp := length(a)+1;
    b:= copy(a,1,pp-1);
    val(b,cd,c);
    sendbyte2(comport,cd);
    a:=copy(a,pp,length(a));
  end;



end;

procedure SClose;
  begin
    if active[comport] then releasecom(comport);
    writeln('Serial Comport Closed.');
  end;

begin

end.
