unit pcomlcd;
{ Parallel port - Serial LCD Unit
  Written on 4/11/00

}
interface
procedure PSInitLCD;
procedure PSClear;
procedure PSCursorOn;
procedure PSCursorOff;
procedure PSHome;
procedure PSPushChar(ch:char);
procedure PSCursor(l,x,lcdlen:byte);
procedure PSerialReadSettings(section:string);
procedure PSDefineChar(cn,d1,d2,d3,d4,d5,d6,d7,d8:byte);

implementation
uses crt,config,dos;

type
 PinStatus = byte;
 PinData = array[1..17] of PinStatus;

 const
   PinDataOn:Pindata = (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0);
   PinDataOff:pindata = (1,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,8);

var
    controldelay:word;
    chardelay:word;
    portaddress:word;
    data:word;
    status:word;
    control:word;

  CommandChar,
  CursorHome,
HideCursor,
ShowCursor,
startrow,startcolumn,
Formfeed,
gotoxyoffset,
Backlighton,
Backlightoff,
blinkon,blinkoff,
stopbits,
cmdgotoxy,
CCOffset1,
CCOffset2,
CustomCharStart: byte;
customchar:string[12];
UseCustom:boolean;
InitString:string[40];

       setup:boolean;
       i:integer;
port2_data:byte;



procedure error(text:string);
begin
  writeln('UART Error: ',text);
end;


{file://Port 2 is write-only. This is a readable copy of what was last written so
file://we can set or clear single bits.
/////////////////////////////////////////////////////////////////////////////}
procedure SCK_High;
  begin
    port2_data := port2_data and (not 1);
    port[control] := port2_data;
{  port2_data&=~0x01;}
  end;

procedure SCK_Low;
  begin
  port2_data := port2_data or 1;
  port[control] := port2_data;
  end;

procedure CS0_High;
  begin
    port[data] := 1;
  end;
procedure CS0_Low;
  begin
    port[data] := 0;
  end;

procedure Data_High;
  begin
{    port2_data&=~0x02;}
     port2_data := port2_data and (not 2);
  port[control] := port2_data;
  end;

procedure Data_Low;
  begin
  {  port2_data|=0x02;}
     port2_data := port2_data or 2;
    port[control] := port2_data;
  end;

procedure Delay_55us;
  var k:word;
  begin
    for k := 1 to controldelay do;
  end;


procedure send_data(data_to_send:byte);
 var
    data_mask:byte;
  begin

  CS0_Low;

  Delay_55us;

  data_mask := $80;
  while (data_mask <> 0) do
{  for(data_mask=0x080;
      data_mask;
      data_mask>>=1)}
    begin
      if ((data_to_send and data_mask) = 1) then
{      if(data_to_send&data_mask)}
        Data_High
      else
        Data_Low;

      Delay_55us;
      SCK_High;
      Delay_55us;
      SCK_Low;
      Delay_55us;
      data_mask := data_mask shr 1;
    end;

  CS0_High;
  Delay_55us;
  end;



function strr(b:longint):string;
  var
   S: string[11];
   begin
    Str(b, S);
    strr := S;
   end;

procedure senddata(b:byte);
var
  a:char;
  s:string;
  u:integer;
begin
  s:=strr(b);
  while(length(s)<3) do s:='0'+s;
  s:='\'+s;
  for u := 1 to length(s) do send_data(ord(s[u]));
end;

procedure sendcommand;
begin
  if commandchar <> 0 then begin
    senddata(commandchar);
  end;
end;


procedure sendbyte(b:byte);              { Byte senden }
begin
  sendcommand;
  senddata(b);
end;

procedure sendbyte2(b:byte);              { Byte senden }
begin
  senddata(b);
end;


procedure sendchar(b:byte);              { Byte senden }
begin
    if ((b >= 0) and (b <= 7)) then
      b:=b+customcharstart;
    send_data(b);
end;



procedure PSPushChar(ch:char);
  begin
    sendchar(ord(ch));
  end;

procedure PSDefineChar(cn,d1,d2,d3,d4,d5,d6,d7,d8:byte);
var
 a:string;
       b:string;
      cd:byte;
      c:integer;
      pp:byte;

  begin
    if UseCustom then
    begin
      sendcommand;

      a:='#'+customchar;

      while (a[1] = '#') do
      begin
        a:=copy(a,2,length(a));
        pp := pos('#',a);
        if pp = 0 then pp := length(a)+1;
        b:= copy(a,1,pp-1);
        val(b,cd,c);
{        writeln('Send ',cd);}
        sendbyte2(cd);
        delay(10);
        a:=copy(a,pp,length(a));
      end;
{      readln;}

{      sendbyte2(comport,customchar);  { Define Char }

      sendbyte2(CCOffset1+cn);
      sendbyte2(CCOffset2+d1);
      sendbyte2(CCOffset2+d2);
      sendbyte2(CCOffset2+d3);
      sendbyte2(CCOffset2+d4);
      sendbyte2(CCOffset2+d5);
      sendbyte2(CCOffset2+d6);
      sendbyte2(CCOffset2+d7);
      sendbyte2(CCOffset2+d8);
    end;
  end;


procedure PSCursor(l,x,lcdlen:byte);
  begin
    if gotoxyoffset <> 0 then
    begin
      sendcommand;
      sendbyte2(cmdgotoxy);
      sendbyte2(gotoxyoffset +((lcdlen)*(l-startrow))+(x-startcolumn));
    end else
    begin
      sendcommand;
      sendbyte2(cmdgotoxy);
      sendbyte2(x-1+startcolumn);
      sendbyte2(l-1+startrow);
    end;
  end;

procedure LPTSetPort(p:word); begin portaddress:=p; end;
procedure PSerialReadSettings(section:string);
var lptport:word;
begin
    ControlDelay := ReadWord(section,'ControlDelay');
    LPTPort := ReadWord(section,'LPTPort');
    if LPTPort <> 0 then LPTSetPort(LPTPort);
    DATA := PORTADDRESS+0;
    STATUS := PORTADDRESS+1;
    CONTROL := PORTADDRESS+2;

  UseCustom := ReadBoolean(section,'DisableCustomChar');
  UseCustom := not UseCustom;
  if stopbits = 0 then stopbits := 1;
  InitString := ReadString(section,'InitString');

  CommandChar:=ReadWord(section,'CommandChar');
  gotoxyoffset:=readword(section,'GotoXYOffset');
  cmdgotoxy:=readword(section,'GotoXY');
  CursorHome:=ReadWord(section,'CursorHome');
  StartRow:=ReadWord(section,'StartRow');
  StartColumn:=ReadWord(section,'StartColumn');
  if gotoxyoffset <> 0 then
  begin
    if StartRow = 0 then StartRow := 1 else Startrow := 0;
    if StartColumn = 0 then StartColumn := 1 else StartColumn := 0;
  end;
  blinkon:=readword(section,'blinkOn');
  blinkoff:=readword(section,'blinkoff');
  HideCursor:=ReadWord(section,'HideCursor');
  ShowCursor:=ReadWord(section,'ShowCursor');
  Formfeed:=ReadWord(section,'ClearScreen');
  Backlighton:=Readword(section,'Backlighton');
  Backlightoff:=ReadWord(section,'Backlightoff');
  CustomCharStart:=ReadWord(section,'CustomCharStart');
  CCOffset1 := ReadWord(section,'CustomCharOffset1');
  CCOffset2 := ReadWord(section,'CustomCharOffset2');
  CustomChar:=ReadString(section,'CustomChar');
end;


function strs(l:longint):string;
var s : string;
begin
  str(l,s);
  strs:=s;
end;


procedure PSClear;
  begin
    sendbyte(formfeed);
  end;

procedure PSCursorOn;
  begin
    sendbyte(ShowCursor);
    sendbyte(BlinkOn);
  end;

procedure PSCursorOff;
  begin
    sendbyte(HideCursor);
    sendbyte(BlinkOff);
  end;

procedure PSHome;
  begin
    sendbyte(CursorHome);
  end;

procedure PSInitLCD;
  var
      a:string;
      b:string;
      cd:byte;
      c:integer;
      pp:byte;
begin

  a := InitString;
  while (a[1] = '#') do
  begin
    a:=copy(a,2,length(a));
    pp := pos('#',a);
    if pp = 0 then pp := length(a)+1;
    b:= copy(a,1,pp-1);
    val(b,cd,c);
    sendbyte2(cd);
    a:=copy(a,pp,length(a));
  end;

  CS0_High;

  port2_data:=1;
  port[control] := port2_data;

  Delay_55us;

  sendbyte(4);
{  send_string("\004");}


end;


begin

end.
